@@ -7,13 +7,12 @@ use lib "$Bin/../lib";
 use PubNub::PubSub;
 use Data::Dumper;
 
-my $pubnub = PubNub::PubSub->new();
-
-my $history = $pubnub->history({
+my $pubnub = PubNub::PubSub->new(
     sub_key => $ENV{PUBNUB_SUB_KEY} || 'sub-c-a66b65f2-2d96-11e4-875c-02ee2ddab7fe',
     channel => $ENV{PUBNUB_CHANNEL} || 'sandbox',
-    total => 100
-});
+);
+
+my $history = $pubnub->history(100);
 
 print Dumper(\$history);
 
